@@ -27,12 +27,12 @@
 #include <alsaevent.h>
 #include <alsaqueue.h>
 
+#include <KIO/NetAccess>
 #include <QTextStream>
 #include <QTextCodec>
 #include <QTime>
 #include <QMutex>
 #include <QMutexLocker>
-#include <KIO/NetAccess>
 
 using namespace drumstick;
 
@@ -55,6 +55,7 @@ namespace KMid {
             m_initialTempo(0),
             m_playlistIndex(-1),
             m_tempoFactor(1.0),
+            m_lastTempo(0),
             m_tick(0),
             m_duration(0),
             m_lastBeat(0),
@@ -68,6 +69,7 @@ namespace KMid {
             for(int i=0; i<MIDI_CHANNELS; ++i) {
                 m_channelUsed[i] = false;
                 m_channelEvents[i] = 0;
+                m_channelPatches[i] = -1;
             }
         }
 
@@ -108,6 +110,7 @@ namespace KMid {
         int m_initialTempo;
         int m_playlistIndex;
         qreal m_tempoFactor;
+        qreal m_lastTempo;
         qint64 m_tick;
         Song m_song;
         QStringList m_loadingMessages;
@@ -126,6 +129,7 @@ namespace KMid {
         int m_channelEvents[MIDI_CHANNELS];
         QByteArray m_trackLabel;
         QByteArray m_channelLabel[MIDI_CHANNELS];
+        int m_channelPatches[MIDI_CHANNELS];
     };
 
     ALSAMIDIObject::ALSAMIDIObject(QObject *parent) : MIDIObject(parent),
@@ -188,8 +192,14 @@ namespace KMid {
         if ( !SequencerEvent::isConnectionChange(ev) &&
              (d->m_state == PlayingState) )
             switch(ev->getSequencerType()) {
-            case SND_SEQ_EVENT_ECHO:
-                emit tick(ev->getTick());
+            case SND_SEQ_EVENT_ECHO: {
+                    emit tick(ev->getTick());
+                    qreal rtempo = currentTempo();
+                    if (rtempo != d->m_lastTempo) {
+                        emit tempoChanged(rtempo);
+                        d->m_lastTempo = rtempo;
+                    }
+                }
                 break;
             case SND_SEQ_EVENT_USR8:
                 emit beat(ev->getRaw32(0), ev->getRaw8(4), ev->getRaw8(5));
@@ -197,13 +207,6 @@ namespace KMid {
             case SND_SEQ_EVENT_TIMESIGN:
                 emit timeSignatureChanged(ev->getRaw8(0), ::pow(2, ev->getRaw8(1)));
                 break;
-            case SND_SEQ_EVENT_TEMPO: {
-                    qreal rtempo = 0.0;
-                    const TempoEvent* te = static_cast<const TempoEvent*>(ev);
-                    if (te != NULL) rtempo = 6e7 / te->getValue() * d->m_tempoFactor;
-                    emit tempoChanged(rtempo);
-                }
-                break;
             case SND_SEQ_EVENT_USR_VAR0: {
                     VariableEvent *ve = static_cast<VariableEvent*>(ev);
                     if (ve != NULL && ve->getData() != NULL && ve->getLength() > 0) {
@@ -414,7 +417,9 @@ namespace KMid {
                     d->setQueueTempo();
                 d->m_out->sendResetMessage();
                 d->m_out->resetControllers();
+                sendInitialProgramChanges();
                 d->m_lastBeat = 0;
+                d->m_lastTempo = 0;
             }
             d->m_player->start();
             updateState( PlayingState );
@@ -555,6 +560,8 @@ namespace KMid {
     {
         d->m_channelUsed[chan] = true;
         d->m_channelEvents[chan]++;
+        if (d->m_channelPatches[chan] < 0)
+            d->m_channelPatches[chan] = patch;
         SequencerEvent* ev = new ProgramChangeEvent (chan, patch);
         appendEvent(ev);
     }
@@ -678,15 +685,15 @@ namespace KMid {
                 d->m_channelUsed[i] = false;
                 d->m_channelEvents[i] = 0;
                 d->m_channelLabel[i].clear();
+                d->m_channelPatches[i] = -1;
             }
             try {
                 d->m_engine->readFromFile(tmpFile);
                 if (!d->m_song.isEmpty()) {
                     d->m_song.sort();
                     addSongPadding();
-                    if (d->m_initialTempo == 0) {
+                    if (d->m_initialTempo == 0)
                         d->m_initialTempo = 500000;
-                    }
                     d->m_song.setFileName(fileName);
                     d->m_player->setSong(&d->m_song);
                     d->setQueueTempo();
@@ -826,6 +833,44 @@ namespace KMid {
         return QString();
     }
 
+    QVariant ALSAMIDIObject::songProperty(const QString& key)
+    {
+        if (key == QLatin1String("SMF_FORMAT"))
+            return QVariant(d->m_song.getFormat());
+        else if (key == QLatin1String("SMF_TRACKS"))
+            return QVariant(d->m_song.getTracks());
+        else if (key == QLatin1String("SMF_DIVISION"))
+            return QVariant(d->m_song.getDivision());
+        else if (key == QLatin1String("NUM_BARS"))
+            return QVariant(d->m_barCount);
+        else if (key == QLatin1String("NUM_BEATS")) {
+            int beats = d->m_song.last()->getTick() / d->m_song.getDivision();
+            return QVariant(beats);
+        }
+        return QVariant();
+    }
+
+    QVariant ALSAMIDIObject::channelProperty(int channel, const QString& key)
+    {
+        if (channel >= 0 && channel < MIDI_CHANNELS) {
+            if (key == QLatin1String("INITIAL_PATCH"))
+                return QVariant(d->m_channelPatches[channel]);
+            else if (key == QLatin1String("LABEL"))
+                return QVariant(d->m_channelLabel[channel]);
+            else if (key == QLatin1String("USED"))
+                return QVariant(d->m_channelUsed[channel]);
+        }
+        return QVariant();
+    }
+
+    void ALSAMIDIObject::sendInitialProgramChanges()
+    {
+        for (int i = 0; i < MIDI_CHANNELS; ++i) {
+            int patch(d->m_channelPatches[i]);
+            d->m_out->sendInitialProgram(i, patch);
+        }
+    }
+
 }
 
 #include "alsamidiobject.moc"
