@@ -851,7 +851,7 @@ void QWrk::processMeterChunk()
         readGap(4);
         int measure = read16bit();
         int  num = readByte();
-        int  den = powl(2, readByte());
+        int  den = pow(2, readByte());
         readGap(4);
         Q_EMIT signalWRKTimeSig(measure, num, den);
     }
@@ -863,7 +863,7 @@ void QWrk::processMeterKeyChunk()
     for (int i = 0; i < count; ++i) {
         int measure = read16bit();
         int  num = readByte();
-        int  den = powl(2, readByte());
+        int  den = pow(2, readByte());
         qint8 alt = readByte();
         Q_EMIT signalWRKTimeSig(measure, num, den);
         Q_EMIT signalWRKKeySig(measure, alt);
@@ -1066,11 +1066,12 @@ void QWrk::processNewTrack()
     Q_EMIT signalWRKNewTrack(name, track, channel, key, vel, port, selected, muted, loop);
     if (bank > -1)
         Q_EMIT signalWRKTrackBank(track, bank);
-    if (patch > -1)
+    if (patch > -1) {
         if (channel > -1)
             Q_EMIT signalWRKProgram(track, 0, channel, patch);
         else
             Q_EMIT signalWRKTrackPatch(track, patch);
+    }
 }
 
 void QWrk::processSoftVer()
