@@ -97,7 +97,9 @@ namespace KMid {
             m_duration(0),
             m_division(0),
             m_songEmpty(true),
-            m_tickInterval(0)
+            m_tickInterval(0),
+            m_numtracks(0),
+            m_barCount(0)
         { }
 
         virtual ~MacMIDIObjectPrivate()
@@ -133,12 +135,15 @@ namespace KMid {
         int m_highestMidiNote;
         bool m_channelUsed[MIDI_CHANNELS];
         QByteArray m_channelLabel[MIDI_CHANNELS];
+        int m_channelPatches[MIDI_CHANNELS];
         QMutex m_openMutex;
         MusicTimeStamp m_sequenceLength;
         qreal m_duration;
         int m_division;
         bool m_songEmpty;
         qint32 m_tickInterval;
+        int m_numtracks;
+        int m_barCount;
         QMap<TextType, TimeStampedData> m_text;
         QMap<MusicTimeStamp, QByteArray> m_textUserEvents;
         QMap<MusicTimeStamp, qreal> m_tempoUserEvents;
@@ -380,6 +385,7 @@ namespace KMid {
             if (currentTime() == 0) {
                 d->m_output->sendResetMessage();
                 d->m_output->resetControllers();
+                sendInitialProgramChanges();
             }
             result = MusicPlayerStart ( d->m_player );
             if (result == noErr)
@@ -722,6 +728,11 @@ namespace KMid {
             case kMusicEventType_MIDIChannelMessage: {
                 MIDIChannelMessage* msgp = (MIDIChannelMessage*) dataPtr;
                 quint8 channel = msgp->status & 0x0f;
+                quint8 type = msgp->status & 0xf0;
+                if ( (type == MIDI_STATUS_PROGRAMCHANGE) &&
+                     (d->m_channelPatches[channel] < 0) ) {
+                    d->m_channelPatches[channel] = msgp->data1;
+                }
                 d->m_channelUsed[channel] = true;
                 channelEvents[channel]++;
                 break;
@@ -752,6 +763,8 @@ namespace KMid {
         OSStatus result = noErr;
         UInt32 ntracks;
         Float64 lengthSeconds;
+        d->m_numtracks = 0;
+        d->m_barCount = 0;
         d->m_duration = 0;
         d->m_sequenceLength = 0;
         d->m_lowestMidiNote = 127;
@@ -763,11 +776,13 @@ namespace KMid {
         for(int i=0; i<MIDI_CHANNELS; ++i) {
             d->m_channelUsed[i] = false;
             d->m_channelLabel[i].clear();
+            d->m_channelPatches[i] = -1;
         }
 
         result = MusicSequenceGetTrackCount (d->m_sequence, &ntracks);
         if (result != noErr)
             kDebug() << "MusicSequenceGetTrackCount() err:" << result;
+        d->m_numtracks = result;
         analyzeTrack(-1); // tempoTrack
         for (UInt32 i = 0; i < ntracks; ++i)
             analyzeTrack(i);
@@ -946,6 +961,7 @@ namespace KMid {
             br.beat = 1;
             br.max = ts.num;
             while ( bTime < d->m_sequenceLength ) {
+                d->m_barCount = br.bar;
                 d->m_beatUserEvents.insert(bTime, br);
                 userEv.data[0] = UserEventBeat;
                 result = MusicTrackNewUserEvent ( userTrack, bTime, &userEv );
@@ -964,7 +980,6 @@ namespace KMid {
                     blen = 4.0 / ts.den;
                 }
             }
-
             bTime = 0;
             userEv.data[0] = UserEventTick;
             double interval = double(d->m_tickInterval) / d->m_division;
@@ -1023,6 +1038,43 @@ namespace KMid {
             break;
         }
     }
+
+    QVariant MacMIDIObject::songProperty(const QString& key)
+    {
+        if (key == QLatin1String("SMF_FORMAT"))
+            return QVariant(1);
+        else if (key == QLatin1String("SMF_TRACKS"))
+            return QVariant(d->m_numtracks);
+        else if (key == QLatin1String("SMF_DIVISION"))
+            return QVariant(d->m_division);
+        else if (key == QLatin1String("NUM_BARS"))
+            return QVariant(d->m_barCount);
+        else if (key == QLatin1String("NUM_BEATS"))
+            return QVariant(trunc(d->m_sequenceLength));
+        return QVariant();
+    }
+
+    QVariant MacMIDIObject::channelProperty(int channel, const QString& key)
+    {
+        if (channel >= 0 && channel < MIDI_CHANNELS) {
+            if (key == QLatin1String("INITIAL_PATCH"))
+                return QVariant(d->m_channelPatches[channel]);
+            else if (key == QLatin1String("LABEL"))
+                return QVariant(d->m_channelLabel[channel]);
+            else if (key == QLatin1String("USED"))
+                return QVariant(d->m_channelUsed[channel]);
+        }
+        return QVariant();
+    }
+
+    void MacMIDIObject::sendInitialProgramChanges()
+    {
+        for (int i = 0; i < MIDI_CHANNELS; ++i) {
+            int patch(d->m_channelPatches[i]);
+            d->m_output->sendInitialProgram(i, patch);
+        }
+    }
+
 }
 
 #include "macmidiobject.moc"
