@@ -44,6 +44,7 @@ namespace KMid {
                 m_volume[chan] = 100;
                 m_muted[chan] = false;
                 m_locked[chan] = false;
+                m_lockedpgm[chan] = 0;
             }
         }
 
@@ -96,9 +97,9 @@ namespace KMid {
 
         void transformProgramEvent(MIDIPacket *packet)
         {
+            int channel = packet->data[0] & 0x0f;
+            m_lastpgm[channel] = packet->data[1];
             if (m_mapper != NULL && m_mapper->isOK()) {
-                int channel = packet->data[0] & 0x0f;
-                m_lastpgm[channel] = packet->data[1];
                 int pgm = m_mapper->patch(channel, m_lastpgm[channel]);
                 if (pgm >= 0 && pgm < 128)
                     packet->data[1] = pgm;
@@ -162,6 +163,7 @@ namespace KMid {
         int m_volume[MIDI_CHANNELS];
         bool m_muted[MIDI_CHANNELS];
         bool m_locked[MIDI_CHANNELS];
+        int m_lockedpgm[MIDI_CHANNELS];
         QByteArray m_resetMessage;
     };
 
@@ -205,7 +207,6 @@ namespace KMid {
 
     int MacMIDIOutput::outputDevice() const
     {
-
         return d->m_outputDevices.indexOf(d->m_currentOutput);
     }
 
@@ -364,6 +365,8 @@ namespace KMid {
         if (channel >= 0 && channel < MIDI_CHANNELS) {
             if (d->m_locked[channel] != lock) {
                 d->m_locked[channel] = lock;
+                if (lock)
+                    d->m_lockedpgm[channel] = d->m_lastpgm[channel];
                 emit lockedChanged( channel, lock );
             }
         }
@@ -566,7 +569,8 @@ namespace KMid {
             sendEvents(pktlist);
     }
 
-    void MacMIDIOutput::sendEvents(const MIDIPacketList* events)
+    void MacMIDIOutput::sendEvents( const MIDIPacketList* events,
+                                    bool discardable )
     {
         quint8 buf[4096];
         MIDIPacketList* pktlist = (MIDIPacketList*) &buf;
@@ -577,13 +581,13 @@ namespace KMid {
             d->transform(&dstpacket);
             quint8 status = dstpacket.data[0] & 0xf0;
             quint8 chan = dstpacket.data[0] & 0x0f;
-            bool ok = ( status >= MIDI_STATUS_NOTEOFF ) &&
-                      ( ( status == MIDI_STATUS_SYSEX ) |
-                        ( status < MIDI_STATUS_SYSEX &&
-                          !d->m_muted[chan] ) );
-            if ( ok && status == MIDI_STATUS_PROGRAMCHANGE )
-                ok = !d->m_locked[chan];
-            if (ok)
+            bool reject = ( status < MIDI_STATUS_NOTEOFF ) ||
+                          ( status >= MIDI_STATUS_SYSEX ) ||
+                          ((status < MIDI_STATUS_SYSEX) &&
+                            discardable && d->m_muted[chan]);
+            if ( discardable && status == MIDI_STATUS_PROGRAMCHANGE )
+                 reject |= d->m_locked[chan];
+            if (!reject)
                curpacket = MIDIPacketListAdd(pktlist, sizeof(buf), curpacket, 0,
                            dstpacket.length, (const Byte*)&dstpacket.data);
             srcpacket = MIDIPacketNext(srcpacket);
@@ -591,6 +595,22 @@ namespace KMid {
         MIDISend(d->m_outPort, d->m_destination, pktlist);
     }
 
+    void MacMIDIOutput::sendInitialProgram(int chan, int program)
+    {
+        int pgm(d->m_locked[chan] ? d->m_lockedpgm[chan] : program);
+        if (pgm > -1) {
+            quint8 data[2];
+            MIDIPacketList pktlist ;
+            MIDIPacket* packet = MIDIPacketListInit(&pktlist);
+            data[0] = MIDI_STATUS_PROGRAMCHANGE | (chan & 0x0f);
+            data[1] = pgm;
+            packet = MIDIPacketListAdd(&pktlist, sizeof(pktlist), packet, 0,
+                sizeof(data), data);
+            if (packet != NULL)
+                sendEvents(&pktlist, false);
+        }
+    }
+
 }
 
 #include "macmidioutput.moc"
