@@ -26,6 +26,9 @@
 #include "rhythmview.h"
 #include "timelabel.h"
 #include "playlistdialog.h"
+#include "kmidadaptor.h"
+
+#include "kmid2.moc"
 
 #include <algorithm>
 #include <QtGui/QDropEvent>
@@ -75,6 +78,8 @@ KMid2::KMid2() : KXmlGuiWindow(),
       m_midiout(0),
       m_settings(new Settings)
 {
+    (void) new KMidAdaptor(this);
+    QDBusConnection::sessionBus().registerObject(QLatin1String("/KMid"), this);
     setAcceptDrops(true);
     setupDockWidgets();
     setupActions();
@@ -97,8 +102,8 @@ void KMid2::slotLoaded(Backend *backend, const QString& library, const QString&
     m_backends.append(midiBackend);
     backend->setParent(this);
     kDebug() << library << name << backend->initialized();
-    if ( backend != NULL && backend->initialized() &&
-         m_currentBackend == NULL &&
+    if ( backend != 0 && backend->initialized() &&
+         m_currentBackend == 0 &&
          (m_settings->midi_backend().isEmpty() ||
           m_settings->midi_backend() == library) ) {
         m_midiobj = backend->midiObject();
@@ -116,6 +121,34 @@ void KMid2::slotLoaded(Backend *backend, const QString& library, const QString&
                 SLOT(slotTimeSignatureEvent(int,int)));
         connect(m_midiobj, SIGNAL(beat(int,int,int)),
                 SLOT(slotBeat(int,int,int)));
+        connect(m_midiobj, SIGNAL(tempoChanged(qreal)),
+                SLOT(slotTempoChanged(qreal)));
+
+        /* DBus signals from MIDIObject ones */
+        connect( m_midiobj, SIGNAL(midiText(int,const QString&)),
+                 SIGNAL(midiTextEvent(int,const QString&)) );
+        connect( m_midiobj, SIGNAL(finished()), SIGNAL(playerFinished()) );
+        connect( m_midiobj, SIGNAL(currentSourceChanged(QString)),
+                 SIGNAL(sourceChanged(QString)) );
+        connect( m_midiobj, SIGNAL(timeSignatureChanged(int,int)),
+                 SIGNAL(timeSignatureEvent(int,int)) );
+        connect( m_midiobj, SIGNAL(beat(int,int,int)),
+                 SIGNAL(beat(int,int,int)) );
+        connect( m_midiobj, SIGNAL(midiNoteOn(int,int,int)),
+                 SIGNAL(midiNoteOnEvent(int,int,int)) );
+        connect( m_midiobj, SIGNAL(midiNoteOff(int,int,int)),
+                 SIGNAL(midiNoteOffEvent(int,int,int)) );
+        connect( m_midiobj, SIGNAL(midiController(int,int,int)),
+                 SIGNAL(midiControllerEvent(int,int,int)) );
+        connect( m_midiobj, SIGNAL(midiKeyPressure(int,int,int)),
+                 SIGNAL(midiKeyPressureEvent(int,int,int)) );
+        connect( m_midiobj, SIGNAL(midiProgram(int,int)),
+                 SIGNAL(midiProgramEvent(int,int)) );
+        connect( m_midiobj, SIGNAL(midiChannelPressure(int,int)),
+                 SIGNAL(midiChannelPressureEvent(int,int)) );
+        connect( m_midiobj, SIGNAL(midiPitchBend(int,int)),
+                 SIGNAL(midiPitchBendEvent(int,int)) );
+
         if (backend->hasSoftSynths())
             backend->initializeSoftSynths(m_settings);
         m_currentBackend = backend;
@@ -132,7 +165,7 @@ void KMid2::initialize()
     connect(m_loader, SIGNAL(loaded(Backend*,const QString&,const QString&)),
                       SLOT(slotLoaded(Backend*,const QString&,const QString&)));
     m_loader->loadAllBackends();
-    if (m_currentBackend == NULL) {
+    if (m_currentBackend == 0) {
         KMessageBox::error(this, i18nc("@info","No MIDI backend loaded."),
                 i18nc("@title:window","Fatal"));
         close();
@@ -193,13 +226,17 @@ void KMid2::initialize()
             m_pianola, SLOT(slotLabel(int,const QString&)),
             Qt::QueuedConnection);
 
-    if ( m_currentBackend != NULL && m_currentBackend->hasSoftSynths()) {
+    if ( m_currentBackend != 0 && m_currentBackend->hasSoftSynths()) {
         connect( m_currentBackend, SIGNAL(softSynthStarted(const QString&,const QStringList&)),
                  SLOT(slotSoftSynthStarted(const QString&,const QStringList&)));
         connect( m_currentBackend, SIGNAL(softSynthErrors(const QString&,const QStringList&)),
                  SLOT(slotSoftSynthErrors(const QString&,const QStringList&)));
     }
 
+    Qt::DockWidgetArea area = dockWidgetArea(m_volDock);
+    if (area != Qt::NoDockWidgetArea)
+        slotDockVolLocationChanged(area);
+
     finished();
     if (!m_settings->exec_fluid() && !m_settings->exec_timidity())
         connectMidiOutput();
@@ -229,7 +266,7 @@ void KMid2::setupActions()
     m_play = new KAction(this);
     m_play->setText(i18nc("@action player play", "Play") );
     m_play->setIcon(KIcon("media-playback-start"));
-    m_play->setShortcut( Qt::Key_P );
+    m_play->setShortcut( Qt::Key_MediaPlay );
     m_play->setWhatsThis(i18nc("@info:whatsthis","Start playback of the current session"));
     m_play->setEnabled(false);
     connect(m_play, SIGNAL(triggered()), SLOT(play()));
@@ -246,6 +283,7 @@ void KMid2::setupActions()
     m_next = new KAction(this);
     m_next->setText(i18nc("@action player next song", "Next song") );
     m_next->setIcon(KIcon("media-skip-forward"));
+    m_next->setShortcut( Qt::Key_MediaNext );
     m_next->setWhatsThis(i18nc("@info:whatsthis","Next song"));
     m_next->setEnabled(false);
     connect(m_next, SIGNAL(triggered()), SLOT(next()));
@@ -254,6 +292,7 @@ void KMid2::setupActions()
     m_previous = new KAction(this);
     m_previous->setText(i18nc("@action player previous song", "Previous song") );
     m_previous->setIcon(KIcon("media-skip-backward"));
+    m_previous->setShortcut( Qt::Key_MediaPrevious );
     m_previous->setWhatsThis(i18nc("@info:whatsthis","Previous song"));
     m_previous->setEnabled(false);
     connect(m_previous, SIGNAL(triggered()), SLOT(previous()));
@@ -278,7 +317,7 @@ void KMid2::setupActions()
     m_stop = new KAction(this);
     m_stop->setText(i18nc("@action player stop", "Stop") );
     m_stop->setIcon(KIcon("media-playback-stop"));
-    m_stop->setShortcut( Qt::Key_S );
+    m_stop->setShortcut( Qt::Key_MediaStop );
     m_stop->setWhatsThis(i18nc("@info:whatsthis","Stop playback or recording") );
     m_stop->setEnabled(false);
     connect(m_stop, SIGNAL(triggered()), SLOT(stop()));
@@ -374,7 +413,7 @@ void KMid2::fileOpen()
 void KMid2::setPlayList(const KUrl::List &urls)
 {
     QList<QUrl> list;
-    if (!urls.empty() && (m_midiobj != NULL)) {
+    if (!urls.empty() && (m_midiobj != 0)) {
         stop();
         if (m_autoSongSettings->isChecked())
             slotSaveSongSettings();
@@ -401,7 +440,7 @@ void KMid2::dragEnterEvent( QDragEnterEvent * event )
 
 void KMid2::dropEvent( QDropEvent * event )
 {
-    if (event->mimeData()->hasUrls() && (m_midiobj != NULL)) {
+    if (event->mimeData()->hasUrls() && (m_midiobj != 0)) {
         QList<QUrl> urls = event->mimeData()->urls();
         if (!urls.empty()) {
             QStringList list;
@@ -432,7 +471,7 @@ void KMid2::slotSourceChanged(const QString &src)
     updateTempoLabel();
     m_seekamt = m_midiobj->totalTime() / 10;
     m_seeking = false;
-    if (m_pianola != NULL) {
+    if (m_pianola != 0) {
         int loNote = m_midiobj->lowestMidiNote();
         int hiNote = m_midiobj->highestMidiNote();
         m_pianola->setNoteRange(loNote, hiNote);
@@ -441,13 +480,13 @@ void KMid2::slotSourceChanged(const QString &src)
             m_pianola->slotLabel(i, m_midiobj->channelLabel(i));
         }
     }
-    if (m_channels != NULL)
+    if (m_channels != 0)
         for(int i = 0; i < MIDI_CHANNELS; ++i ) {
-            if (m_midiobj->channelUsed(i))
-                m_channels->setLockChannel(i, false);
+            m_midiout->setLocked(i, false);
+            m_midiout->setMuted(i, false);
+            m_channels->setLockChannel(i, false);
             m_channels->enableChannel(i, m_midiobj->channelUsed(i));
-            if (m_midiobj->channelUsed(i))
-                m_channels->setChannelName(i, m_midiobj->channelLabel(i));
+            m_channels->setChannelName(i, m_midiobj->channelLabel(i));
         }
     if (m_autoSongSettings->isChecked())
         slotLoadSongSettings();
@@ -469,7 +508,7 @@ void KMid2::displayLyrics()
                      KGlobal::charsets()->descriptiveEncodingNames() ) {
                 QString codecName = KGlobal::charsets()->encodingForName(name);
                 QTextCodec* codec = QTextCodec::codecForName(codecName.toLatin1());
-                if (codec != NULL && encoding == codec->name() ) {
+                if (codec != 0 && encoding == codec->name() ) {
                     slotSelectEncoding(name);
                     break;
                 }
@@ -523,6 +562,7 @@ void KMid2::slotUpdateState( State newState, State /*oldState*/ )
     default:
         break;
     }
+    emit playerStateChanged(newState);
 }
 
 void KMid2::updateState(const QString &newState, const QString &stateName)
@@ -533,7 +573,7 @@ void KMid2::updateState(const QString &newState, const QString &stateName)
 
 void KMid2::play()
 {
-    if (m_midiobj != NULL) {
+    if (m_midiobj != 0) {
         slotCheckOutput();
         displayLyrics();
         m_midiobj->play();
@@ -543,8 +583,8 @@ void KMid2::play()
 
 void KMid2::pause()
 {
-    if (m_midiobj != NULL) {
-        if (m_pause->isChecked()) {
+    if (m_midiobj != 0) {
+        if (m_midiobj->state() == PlayingState) {
             m_midiobj->pause();
             m_pianola->allNotesOff();
             m_channels->allNotesOff();
@@ -556,18 +596,18 @@ void KMid2::pause()
 
 void KMid2::stop()
 {
-    if (m_midiobj != NULL)
+    if (m_midiobj != 0)
         m_midiobj->stop();
-    if (m_pianola != NULL)
+    if (m_pianola != 0)
         m_pianola->allNotesOff();
-    if (m_channels != NULL)
+    if (m_channels != 0)
         m_channels->allNotesOff();
     m_rtempo = 0;
 }
 
 void KMid2::forward()
 {
-    if (m_midiobj != NULL) {
+    if (m_midiobj != 0) {
         qint64 s = m_midiobj->currentTime() + m_seekamt;
         if (s < m_midiobj->totalTime()) {
             m_timeSlider->setValue(s);
@@ -577,7 +617,7 @@ void KMid2::forward()
 
 void KMid2::rewind()
 {
-    if (m_midiobj != NULL) {
+    if (m_midiobj != 0) {
         qint64 s = m_midiobj->currentTime() - m_seekamt;
         if (s > 0) {
             m_timeSlider->setValue(s);
@@ -590,7 +630,7 @@ void KMid2::finished()
     if (m_autoSongSettings->isChecked())
         slotSaveSongSettings();
     m_timeSlider->setValue(0);
-    if ((m_midiobj != NULL) && m_loop->isChecked()) {
+    if ((m_midiobj != 0) && m_loop->isChecked()) {
         QStringList queue = m_midiobj->queue();
         if ((queue.count() > 0) &&
             (queue.indexOf(m_midiobj->currentSource()) == queue.count()-1))
@@ -604,7 +644,7 @@ void KMid2::slotSelectEncoding(int i)
         m_songEncoding.clear(); // Default
     else
         m_songEncoding = KGlobal::charsets()->encodingForName(m_comboCodecs->itemText(i));
-    if (m_midiobj != NULL)
+    if (m_midiobj != 0)
         m_midiobj->setTextEncoding(m_songEncoding);
     displayLyrics();
 }
@@ -618,7 +658,7 @@ void KMid2::slotSelectEncoding(const QString& encoding)
         m_comboCodecs->setCurrentItem(encoding, false, 0);
         m_songEncoding = KGlobal::charsets()->encodingForName(encoding);
     }
-    if (m_midiobj != NULL)
+    if (m_midiobj != 0)
         m_midiobj->setTextEncoding(m_songEncoding);
 }
 
@@ -652,11 +692,24 @@ void KMid2::slotTempoReset()
     m_tempoSlider->setToolTip("100%");
 }
 
+inline double sliderToTempoFactor(int sliderValue)
+{
+    return (sliderValue*sliderValue + 100.0*sliderValue + 20000.0) / 400.0;
+}
+
 void KMid2::slotTempoSlider(int value)
 {
-    qreal tempoFactor = (value*value + 100.0*value + 20000.0) / 40000.0;
-    m_midiobj->setTimeSkew( tempoFactor );
-    if (m_midiobj->state() != PlayingState) updateTempoLabel();
+    m_midiobj->setTimeSkew( sliderToTempoFactor(value)/100.0 );
+    if (m_midiobj->state() != PlayingState)
+        updateTempoLabel();
+    slotTempoSliderMoved(value);
+}
+
+void KMid2::slotTempoSliderMoved(int value)
+{
+    QString tip = QString::number(sliderToTempoFactor(value), 'f', 0) + '%';
+    m_tempoSlider->setToolTip(tip);
+    QToolTip::showText(QCursor::pos(), tip, this);
 }
 
 void KMid2::slotTimeSlider(int value)
@@ -669,24 +722,17 @@ void KMid2::slotTimeSlider(int value)
     m_seeking = false;
 }
 
+void KMid2::slotTimeSliderPressed()
+{
+    m_seeking = true;
+}
+
 void KMid2::slotVolumeSlider(int value)
 {
     m_midiout->setVolume( -1, value*0.01 );
-    if (m_channels != NULL)
+    if (m_channels != 0)
         m_channels->setVolumeFactor( value*0.01 );
-}
-
-void KMid2::slotPitchSlider(int value)
-{
-    m_midiout->setPitchShift( value );
-}
-
-void KMid2::slotTempoSliderMoved(int value)
-{
-    qreal tempoFactor = (value*value + 100.0*value + 20000.0) / 400.0;
-    QString tip = QString::number(tempoFactor, 'f', 0) + '%';
-    m_tempoSlider->setToolTip(tip);
-    QToolTip::showText(QCursor::pos(), tip, this);
+    slotVolumeSliderMoved(value);
 }
 
 void KMid2::slotVolumeSliderMoved(int value)
@@ -696,6 +742,12 @@ void KMid2::slotVolumeSliderMoved(int value)
     QToolTip::showText(QCursor::pos(), tip, this);
 }
 
+void KMid2::slotPitchSlider(int value)
+{
+    m_midiout->setPitchShift( value );
+    slotPitchSliderMoved(value);
+}
+
 void KMid2::slotPitchSliderMoved(int value)
 {
     QString tip = QString::number(value);
@@ -703,17 +755,11 @@ void KMid2::slotPitchSliderMoved(int value)
     QToolTip::showText(QCursor::pos(), tip, this);
 }
 
-void KMid2::slotTick(qint64 tick)
+void KMid2::slotTick(qint64 time)
 {
-    if (!m_seeking) {
-        m_timeSlider->setSliderPosition(tick);
-        updateTempoLabel();
-    }
-}
-
-void KMid2::slotTimeSliderPressed()
-{
-    m_seeking = true;
+    if (!m_seeking)
+        m_timeSlider->setSliderPosition(time);
+    emit tick(time);
 }
 
 void KMid2::slotEditSettings()
@@ -745,7 +791,7 @@ void KMid2::slotEditSettings()
     ui_prefs_midi.kcfg_sysex_file->setEnabled(m_settings->reset_mode() == Settings::Syx);
     dialog->addPage(lyricsSettings, i18nc("@title:group","Lyrics"), "format-text-color");
     dialog->addPage(midiSettings, i18nc("@title:group","MIDI"), "audio-midi");
-    if ( m_currentBackend != NULL && m_currentBackend->hasSoftSynths()) {
+    if ( m_currentBackend != 0 && m_currentBackend->hasSoftSynths()) {
         QWidget *progsSettings = new QWidget(dialog);
         m_currentBackend->setupConfigurationWidget(progsSettings);
         dialog->addPage(progsSettings, i18nc("@title:group","Soft Synths"), "applications-system");
@@ -766,7 +812,7 @@ void KMid2::slotApplySettings(const QString& name)
     Q_UNUSED(name);
     QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
 
-    if ( m_currentBackend != NULL &&
+    if ( m_currentBackend != 0 &&
          ( !m_currentBackend->hasSoftSynths() ||
            !m_currentBackend->applySoftSynthSettings() ))
         connectMidiOutput();
@@ -800,13 +846,13 @@ void KMid2::slotUpdateConfigDialogWidgets()
     ui_prefs_midi.connList->addItems(lst);
     ui_prefs_midi.connList->setCurrentRow(m_midiout->outputDevice());
 
-    if ( m_currentBackend != NULL && m_currentBackend->hasSoftSynths())
+    if ( m_currentBackend != 0 && m_currentBackend->hasSoftSynths())
         m_currentBackend->updateConfigWidget();
 }
 
 void KMid2::slotReadSettings()
 {
-    if ( m_currentBackend != NULL && m_currentBackend->hasSoftSynths())
+    if ( m_currentBackend != 0 && m_currentBackend->hasSoftSynths())
         m_currentBackend->saveSettings();
 
     if (m_recentFiles->urls().isEmpty()) {
@@ -818,7 +864,7 @@ void KMid2::slotReadSettings()
         m_mapper.clear();
     else {
         m_mapper.loadFile(mapperFile);
-        if (m_midiout != NULL) m_midiout->setMidiMap(&m_mapper);
+        if (m_midiout != 0) m_midiout->setMidiMap(&m_mapper);
     }
     switch(m_settings->reset_mode()) {
     case Settings::None:
@@ -843,7 +889,7 @@ void KMid2::slotReadSettings()
         file.close();
         break;
     }
-    if (m_midiout != NULL)
+    if (m_midiout != 0)
         m_midiout->setResetMessage(m_resetMessage);
     m_autoSongSettings->setChecked(m_settings->auto_song_settings());
     slotSelectEncoding(m_settings->encoding());
@@ -870,13 +916,13 @@ void KMid2::slotWriteSettings()
 bool KMid2::queryExit()
 {
     stop();
-    if (m_midiout != NULL) {
+    if (m_midiout != 0) {
         m_midiout->allNotesOff();
         m_midiout->resetControllers();
     }
     if (m_autoSongSettings->isChecked())
         slotSaveSongSettings();
-    if ( m_currentBackend != NULL &&
+    if ( m_currentBackend != 0 &&
          m_currentBackend->hasSoftSynths() )
         m_currentBackend->terminateSoftSynths();
 
@@ -957,6 +1003,8 @@ void KMid2::previous()
     int i = queue.indexOf(m_midiobj->currentSource());
     if (i > 0) {
         stop();
+        if (m_autoSongSettings->isChecked())
+            slotSaveSongSettings();
         m_midiobj->setCurrentSource(queue.at(i-1));
     }
 }
@@ -967,6 +1015,8 @@ void KMid2::next()
     int i = queue.indexOf(m_midiobj->currentSource());
     if (i < queue.count()-1) {
         stop();
+        if (m_autoSongSettings->isChecked())
+            slotSaveSongSettings();
         m_midiobj->setCurrentSource(queue.at(i+1));
     }
 }
@@ -1062,7 +1112,7 @@ void KMid2::slotLoadPlaylist()
     QString fileName = KFileDialog::getOpenFileName(
             KUrl("kfiledialog:///KMid2PlayList"), "text/plain",
             this, i18nc("@title:window","Open playlist from file"));
-    if (!fileName.isEmpty() && (m_midiobj != NULL))
+    if (!fileName.isEmpty() && (m_midiobj != 0))
         loadPlaylist(fileName);
 }
 
@@ -1071,7 +1121,7 @@ void KMid2::slotSavePlaylist()
     QString fileName = KFileDialog::getSaveFileName(
             KUrl("kfiledialog:///KMid2PlayList"), "text/plain",
             this, i18nc("@title:window","Save playlist to file"));
-    if ( !fileName.isEmpty() && (m_midiobj != NULL) ) {
+    if ( !fileName.isEmpty() && (m_midiobj != 0) ) {
         KSaveFile file( fileName );
         file.open();
         QTextStream stream( &file );
@@ -1090,7 +1140,7 @@ void KMid2::slotManagePlaylist()
     dialog->setItems( m_midiobj->queue() );
     dialog->setCurrentItem( m_midiobj->currentSource() );
     if (dialog->exec() == KDialog::Accepted) {
-        if (dialog != NULL) {
+        if (dialog != 0) {
             stop();
             if (dialog->itemCount() > 0) {
                 m_midiobj->setQueue(dialog->items());
@@ -1140,7 +1190,7 @@ void KMid2::slotTimeSignatureEvent(const int numerator, const int denominator )
 
 void KMid2::slotShowPianola(bool checked)
 {
-    if (m_pianola != NULL)
+    if (m_pianola != 0)
         m_pianola->setVisible(checked);
 }
 
@@ -1151,7 +1201,7 @@ void KMid2::slotPianolaClosed()
 
 void KMid2::slotShowChannels(bool checked)
 {
-    if (m_channels != NULL)
+    if (m_channels != 0)
         m_channels->setVisible(checked);
 }
 
@@ -1164,13 +1214,12 @@ void KMid2::setupDockWidgets()
 {
     /* Volume and Pitch dock */
 
-    QDockWidget *dock = new QDockWidget(i18nc("@title:window", "Volume and Pitch"), this);
-    dock->setObjectName(QLatin1String("volumeAndPitchDock"));
-    dock->setAllowedAreas(Qt::LeftDockWidgetArea | Qt::RightDockWidgetArea);
-    dock->setFeatures(QDockWidget::DockWidgetClosable |
+    m_volDock = new QDockWidget(i18nc("@title:window", "Volume and Pitch"), this);
+    m_volDock->setObjectName(QLatin1String("volumeAndPitchDock"));
+    m_volDock->setFeatures(QDockWidget::DockWidgetClosable |
             QDockWidget::DockWidgetFloatable | QDockWidget::DockWidgetMovable |
             QDockWidget::DockWidgetVerticalTitleBar );
-    QWidget *dw = new QWidget(dock);
+    QWidget *dw = new QWidget(m_volDock);
     QGridLayout *gridLayout = new QGridLayout(dw);
 
     QLabel *labelVol = new QLabel(this);
@@ -1205,21 +1254,24 @@ void KMid2::setupDockWidgets()
     gridLayout->addWidget(m_pitchSlider, 1, 1);
     labelPitch->setBuddy(m_pitchSlider);
 
-    dock->setWidget(dw);
-    addDockWidget(Qt::LeftDockWidgetArea, dock);
+    m_volDock->setWidget(dw);
+    addDockWidget(Qt::LeftDockWidgetArea, m_volDock);
 
-    QAction* action = dock->toggleViewAction();
+    QAction* action = m_volDock->toggleViewAction();
     action->setText(i18nc("@title:window", "Volume and Pitch"));
     action->setIcon(KIcon("view-media-equalizer"));
     action->setWhatsThis(i18nc("@action:inmenu","Show or hide the volume and pitch controls"));
     actionCollection()->addAction("show_volume_pitch", action);
 
+    connect ( m_volDock, SIGNAL(dockLocationChanged(Qt::DockWidgetArea)),
+              SLOT(slotDockVolLocationChanged(Qt::DockWidgetArea)));
+
     /* Time Position dock */
 
-    dock = new QDockWidget(i18nc("@title:window", "Time Position"), this);
-    dock->setObjectName(QLatin1String("timeDock"));
-    dock->setAllowedAreas(Qt::TopDockWidgetArea | Qt::BottomDockWidgetArea);
-    dw = new QWidget(dock);
+    m_posDock = new QDockWidget(i18nc("@title:window", "Time Position"), this);
+    m_posDock->setObjectName(QLatin1String("timeDock"));
+    m_posDock->setAllowedAreas(Qt::TopDockWidgetArea | Qt::BottomDockWidgetArea);
+    dw = new QWidget(m_posDock);
     QVBoxLayout* vLayout = new QVBoxLayout(dw);
 
     m_timeSlider = new QSlider(this);
@@ -1237,10 +1289,10 @@ void KMid2::setupDockWidgets()
     m_timeLabel->setMarkers(8);
     vLayout->addWidget(m_timeLabel);
 
-    dock->setWidget(dw);
-    addDockWidget(Qt::TopDockWidgetArea, dock);
+    m_posDock->setWidget(dw);
+    addDockWidget(Qt::TopDockWidgetArea, m_posDock);
 
-    action = dock->toggleViewAction();
+    action = m_posDock->toggleViewAction();
     action->setText(i18nc("@title:window", "Time Position"));
     action->setIcon(KIcon("player-time"));
     action->setWhatsThis(i18nc("@action:inmenu","Show or hide the time position controls"));
@@ -1248,18 +1300,18 @@ void KMid2::setupDockWidgets()
 
     /* Codecs dock */
 
-    dock = new QDockWidget(i18nc("@title:window", "Text Encoding"), this);
-    dock->setObjectName(QLatin1String("codecsDock"));
-    dock->setAllowedAreas(Qt::TopDockWidgetArea | Qt::BottomDockWidgetArea);
+    m_encDock = new QDockWidget(i18nc("@title:window", "Text Encoding"), this);
+    m_encDock->setObjectName(QLatin1String("codecsDock"));
+
     m_comboCodecs = new KComboBox(this);
     m_comboCodecs->setWhatsThis(i18nc("@info:whatsthis","Character encoding for lyrics and other text"));
     m_comboCodecs->addItem(i18nc("@item:inlistbox Default MIDI text encoding", "Default ( ASCII )"));
     m_comboCodecs->addItems( KGlobal::charsets()->descriptiveEncodingNames() );
     m_comboCodecs->setCurrentItem(m_settings->encoding());
-    dock->setWidget(m_comboCodecs);
-    addDockWidget(Qt::TopDockWidgetArea, dock);
+    m_encDock->setWidget(m_comboCodecs);
+    addDockWidget(Qt::TopDockWidgetArea, m_encDock);
 
-    action = dock->toggleViewAction();
+    action = m_encDock->toggleViewAction();
     action->setText(i18nc("@title:window", "Text Encoding"));
     action->setIcon(KIcon("view-list-text"));
     action->setWhatsThis(i18nc("@action:inmenu","Show or hide the character encoding control"));
@@ -1267,10 +1319,10 @@ void KMid2::setupDockWidgets()
 
     /* Rhythm dock */
 
-    dock = new QDockWidget(i18nc("@title:window", "Rhythm"), this);
-    dock->setObjectName(QLatin1String("rhythmDock"));
-    dock->setAllowedAreas(Qt::TopDockWidgetArea | Qt::BottomDockWidgetArea);
-    dw = new QWidget(dock);
+    m_rthmDock = new QDockWidget(i18nc("@title:window", "Rhythm"), this);
+    m_rthmDock->setObjectName(QLatin1String("rhythmDock"));
+    m_rthmDock->setAllowedAreas(Qt::TopDockWidgetArea | Qt::BottomDockWidgetArea);
+    dw = new QWidget(m_rthmDock);
     gridLayout = new QGridLayout(dw);
 
     QLabel* labelSpeed = new QLabel(this);
@@ -1318,10 +1370,10 @@ void KMid2::setupDockWidgets()
     m_rhythm->setWhatsThis(i18nc("@info:whatsthis","Visual metronome"));
     gridLayout->addWidget(m_rhythm, 1, 2, 1, 1, Qt::AlignVCenter);;
 
-    dock->setWidget(dw);
-    addDockWidget(Qt::BottomDockWidgetArea, dock);
+    m_rthmDock->setWidget(dw);
+    addDockWidget(Qt::BottomDockWidgetArea, m_rthmDock);
 
-    action = dock->toggleViewAction();
+    action = m_rthmDock->toggleViewAction();
     action->setText(i18nc("@title:window", "Rhythm"));
     action->setIcon(KIcon("chronometer"));
     action->setWhatsThis(i18nc("@action:inmenu","Show or hide the rhythm controls"));
@@ -1400,6 +1452,8 @@ void KMid2::slotSaveSongSettings()
 
 void KMid2::slotLoadSongSettings()
 {
+    int vol, pitch, skew, pgm;
+    bool locked, muted, solo;
     if (!m_songName.isEmpty()) {
         QString fileName = QString("songlib/%1.cfg").arg(m_songName);
         KConfig songSettings(KStandardDirs::locateLocal("appdata", fileName, false));
@@ -1408,9 +1462,19 @@ void KMid2::slotLoadSongSettings()
         QString encoding = grp.readEntry("encoding", QString());
         if (!encoding.isEmpty())
             slotSelectEncoding(encoding);
-        m_volumeSlider->setValue(grp.readEntry("volume", 100));
-        m_pitchSlider->setValue(grp.readEntry("pitch", 0));
-        m_tempoSlider->setValue(grp.readEntry("timeskew", 100));
+
+        vol = grp.readEntry("volume", 100);
+        m_volumeSlider->setValue(vol);
+        m_volumeSlider->setToolTip(QString::number(vol*1.0, 'f', 0) + '%');
+
+        pitch = grp.readEntry("pitch", 0);
+        m_pitchSlider->setValue(pitch);
+        m_pitchSlider->setToolTip(QString::number(pitch));
+
+        skew = grp.readEntry("timeskew", 100);
+        m_tempoSlider->setValue(skew);
+        m_tempoSlider->setToolTip(QString::number(sliderToTempoFactor(skew),'f',0)+'%');
+
         for(int i = 0; i < MIDI_CHANNELS; ++i ) {
             QString grpName = QString("MIDI Channel %1").arg(i+1,2);
             if ( songSettings.hasGroup(grpName) ) {
@@ -1422,10 +1486,22 @@ void KMid2::slotLoadSongSettings()
                     m_channels->setChannelName(i, name);
                     m_pianola->slotLabel(i, name);
                 }
-                m_channels->setMuteChannel(i, grp.readEntry("muted", false));
-                m_channels->setSoloChannel(i, grp.readEntry("solo", false));
-                m_channels->setPatchChannel(i, grp.readEntry("patch", -1));
-                m_channels->setLockChannel(i, grp.readEntry("locked", false));
+
+                muted = grp.readEntry("muted", false);
+                m_midiout->setMuted(i, muted);
+                m_channels->setMuteChannel(i, muted);
+
+                solo = grp.readEntry("solo", false);
+                m_channels->setSoloChannel(i, solo);
+
+                pgm = grp.readEntry("patch", -1);
+                m_channels->setPatchChannel(i, pgm);
+                if (pgm > -1)
+                    m_midiout->sendProgram(i, pgm);
+
+                locked = grp.readEntry("locked", false);
+                m_channels->setLockChannel(i, locked);
+                m_midiout->setLocked(i, locked);
             }
         }
         displayLyrics();
@@ -1435,9 +1511,12 @@ void KMid2::slotLoadSongSettings()
 void KMid2::slotSoftSynthErrors(const QString& pgm, const QStringList& messages)
 {
     KMessageBox::error(this,
-        i18nc("@info", "Failed to run %1 with the provided "
-        "arguments.<nl/>Returned messages:<nl/>%2",
-        pgm, messages.join("<nl/>")),
+        i18ncp("@info", "Failed to run %2 with the provided "
+                "arguments.<nl/>Returned message:<nl/>%3",
+                "Failed to run %2 with the provided "
+                "arguments.<nl/>Returned messages:<nl/>%3",
+                messages.size(),
+                pgm, messages.join("<nl/>")),
         i18nc("@title:window", "%1 startup failed", pgm));
 }
 
@@ -1460,11 +1539,17 @@ void KMid2::slotSoftSynthStarted(const QString& pgm, const QStringList& messages
 {
     if (!messages.isEmpty())
         KMessageBox::informationList(this,
-            i18nc("@info", "%1 has returned the following messages "
-            "when launched with the provided arguments.", pgm),
+            i18ncp("@info", "%2 has returned the following message "
+                    "when launched with the provided arguments.",
+                    "%2 has returned the following messages "
+                    "when launched with the provided arguments.",
+                    messages.size(),
+                    pgm),
             messages,
-            i18nc("@title:window", "%1 messages", pgm),
-            "timidity_warnings");
+            i18ncp("@title:window", "%2 message", "%2 messages",
+                    messages.size(),
+                    pgm),
+            "softsynth_warnings");
     connectMidiOutput();
 }
 
@@ -1473,9 +1558,9 @@ void KMid2::readProperties(const KConfigGroup &cfg)
     QString currentSource = cfg.readEntry("CurrentSource", QString());
     QString playList = cfg.readEntry("PlayList", QString());
     bool wasPlaying = cfg.readEntry("Playing", false);
-    if (!playList.isEmpty() && (m_midiobj != NULL))
+    if (!playList.isEmpty() && (m_midiobj != 0))
         loadPlaylist(playList);
-    if (playList.isEmpty() && !currentSource.isEmpty() && (m_midiobj != NULL))
+    if (playList.isEmpty() && !currentSource.isEmpty() && (m_midiobj != 0))
         m_midiobj->setCurrentSource(currentSource);
     if (wasPlaying && !m_settings->player_autostart())
         play();
@@ -1505,4 +1590,191 @@ void KMid2::showEvent(QShowEvent* event)
     }
 }
 
-#include "kmid2.moc"
+void KMid2::slotDockVolLocationChanged(Qt::DockWidgetArea area)
+{
+    QDockWidget::DockWidgetFeatures flags = QDockWidget::DockWidgetClosable |
+        QDockWidget::DockWidgetFloatable | QDockWidget::DockWidgetMovable;
+    if (area == Qt::LeftDockWidgetArea || area == Qt::RightDockWidgetArea) {
+        flags |= QDockWidget::DockWidgetVerticalTitleBar;
+        m_volumeSlider->setOrientation(Qt::Vertical);
+        m_volumeSlider->setTickPosition(QSlider::TicksBelow);
+        m_pitchSlider->setOrientation(Qt::Vertical);
+        m_pitchSlider->setTickPosition(QSlider::TicksBelow);
+    } else { // top or bottom
+        m_volumeSlider->setOrientation(Qt::Horizontal);
+        m_volumeSlider->setTickPosition(QSlider::TicksAbove);
+        m_pitchSlider->setOrientation(Qt::Horizontal);
+        m_pitchSlider->setTickPosition(QSlider::TicksAbove);
+    }
+    m_volDock->setFeatures(flags);
+}
+
+/* DBus interface */
+
+void KMid2::reload()
+{
+    if (m_midiobj != 0) {
+        if (m_midiobj->state() == PlayingState)
+            stop();
+        qint64 curpos = m_midiobj->currentTime();
+        QString cursrc = m_midiobj->currentSource();
+        if (!cursrc.isEmpty()) {
+            m_midiobj->clear();
+            m_midiobj->setCurrentSource(cursrc);
+            m_timeSlider->setValue(curpos);
+        }
+    }
+}
+
+QStringList KMid2::metaData(const QString& key)
+{
+    if (m_midiobj != 0)
+        return m_midiobj->metaData(key);
+    return QStringList();
+}
+
+bool KMid2::isMuted(int channel)
+{
+    if (m_midiout != 0)
+        return m_midiout->isMuted(channel);
+    return false;
+}
+
+void KMid2::setMuted(int channel, bool muted)
+{
+    if (m_channels != 0)
+        m_channels->setMuteChannel(channel, muted);
+}
+
+bool KMid2::autoStart()
+{
+    return m_autostart->isChecked();
+}
+
+bool KMid2::isLooping()
+{
+    return m_loop->isChecked();
+}
+
+qlonglong KMid2::length()
+{
+    if (m_midiobj != 0)
+        return m_midiobj->totalTime();
+    return 0;
+}
+
+QString KMid2::midiConnection()
+{
+    if (m_midiout != 0)
+        return m_midiout->outputDeviceName();
+    return QString();
+}
+
+bool KMid2::openUrl(const QString& url)
+{
+    if (m_midiobj != 0) {
+        m_midiobj->setCurrentSource(url);
+        return true;
+    }
+    return false;
+}
+
+qlonglong KMid2::position()
+{
+    if (m_midiobj != 0)
+        return m_midiobj->currentTime();
+    return 0;
+}
+
+void KMid2::seek(qlonglong pos)
+{
+    if ((m_midiobj != 0) && (pos < m_midiobj->totalTime()))
+        m_timeSlider->setValue(pos);
+}
+
+void KMid2::setAutoStart(bool arg)
+{
+    m_autostart->setChecked(arg);
+}
+
+void KMid2::setLooping(bool arg)
+{
+    m_loop->setChecked(arg);
+}
+
+void KMid2::setMidiConnection(const QString& conn)
+{
+    if (m_midiout != 0)
+        m_midiout->setOutputDeviceName(conn);
+}
+
+void KMid2::setTempoFactor(double factor)
+{
+    if ((factor >= .5) && (factor <= 2.0))
+        m_tempoSlider->setValue(50*(sqrt(factor*16-7)-1));
+}
+
+void KMid2::setTranspose(int amount)
+{
+    if ((amount >= -12) && (amount <= 12))
+        m_pitchSlider->setValue(amount);
+}
+
+void KMid2::setVolumeFactor(double factor)
+{
+    if ((factor >= 0) && (factor <= 2.0))
+        m_volumeSlider->setValue(factor * 100);
+}
+
+int KMid2::state()
+{
+    if (m_midiobj != 0)
+        return m_midiobj->state();
+    return ErrorState;
+}
+
+double KMid2::tempoFactor()
+{
+    if (m_midiobj != 0)
+        return m_midiobj->timeSkew();
+    return 1.0;
+}
+
+int KMid2::transpose()
+{
+    if (m_midiout != 0)
+        return m_midiout->pitchShift();
+    return 0;
+}
+
+double KMid2::volumeFactor()
+{
+    return (m_volumeSlider->value() / 100.0);
+}
+
+QString KMid2::currentSource() const
+{
+    if (m_midiobj != 0)
+        return m_midiobj->currentSource();
+    return QString();
+}
+
+void KMid2::slotTempoChanged(qreal tempo)
+{
+    updateTempoLabel();
+    emit tempoEvent(tempo);
+}
+
+QDBusVariant KMid2::songProperty(const QString& key)
+{
+    if (m_midiobj != 0)
+        return QDBusVariant(m_midiobj->songProperty(key));
+    return QDBusVariant();
+}
+
+QDBusVariant KMid2::channelProperty(int channel, const QString& key)
+{
+    if (m_midiobj != 0)
+        return QDBusVariant(m_midiobj->channelProperty(channel, key));
+    return QDBusVariant();
+}
