@@ -20,7 +20,9 @@
 #ifndef VUMETER_H
 #define VUMETER_H
 
-#include <QWidget>
+#include <QtGui/QWidget>
+
+class QImage;
 
 class Vumeter : public QWidget
 {
@@ -39,11 +41,14 @@ public slots:
     void setMaxValue(qreal);
 
 protected:
+    void initImage();
     void paintEvent(QPaintEvent *);
+    void resizeEvent(QResizeEvent *);
     QSize minimumSizeHint() const;
     QSize sizeHint() const;
 
 private:
+    QImage *m_img;
     qreal m_max;
     qreal m_value;
 };
