@@ -90,6 +90,7 @@ namespace KMid {
         bool m_channelUsed[MIDI_CHANNELS];
         int m_channelEvents[MIDI_CHANNELS];
         QByteArray m_channelLabel[MIDI_CHANNELS];
+        int m_channelPatches[MIDI_CHANNELS];
         QByteArray m_trackLabel;
         QMutex m_openMutex;
         QMap<uint,QByteArray> m_textUserEvents;
@@ -277,6 +278,7 @@ namespace KMid {
             d->m_channelUsed[i] = false;
             d->m_channelEvents[i] = 0;
             d->m_channelLabel[i].clear();
+            d->m_channelPatches[i] = -1;
         }
     }
 
@@ -301,6 +303,7 @@ namespace KMid {
                 d->m_out->setInitialTempo(d->m_initialTempo);
             d->m_out->sendResetMessage();
             d->m_out->resetControllers();
+            sendInitialProgramChanges();
             d->m_out->songOutput();
             if (d->m_out->play())
                 updateState( PlayingState );
@@ -496,6 +499,8 @@ namespace KMid {
     {
         d->m_channelUsed[chan] = true;
         d->m_channelEvents[chan]++;
+        if (d->m_channelPatches[chan] < 0)
+            d->m_channelPatches[chan] = patch;
         SongEvent* ev = new SongEvent ( SHORT_EVENT,
             MIDI_STATUS_PROGRAMCHANGE | (chan & MIDI_CHANNEL_MASK), patch);
         appendEvent(ev);
@@ -774,6 +779,44 @@ namespace KMid {
         }
     }
 
+    QVariant WinMIDIObject::songProperty(const QString& key)
+    {
+        if (key == QLatin1String("SMF_FORMAT"))
+            return QVariant(d->m_song.getFormat());
+        else if (key == QLatin1String("SMF_TRACKS"))
+            return QVariant(d->m_song.getTracks());
+        else if (key == QLatin1String("SMF_DIVISION"))
+            return QVariant(d->m_song.getDivision());
+        else if (key == QLatin1String("NUM_BARS"))
+            return QVariant(d->m_barCount);
+        else if (key == QLatin1String("NUM_BEATS")) {
+            int beats = d->m_song.last()->getTick() / d->m_song.getDivision();
+            return QVariant(beats);
+        }
+        return QVariant();
+    }
+
+    QVariant WinMIDIObject::channelProperty(int channel, const QString& key)
+    {
+        if (channel >= 0 && channel < MIDI_CHANNELS) {
+            if (key == QLatin1String("INITIAL_PATCH"))
+                return QVariant(d->m_channelPatches[channel]);
+            else if (key == QLatin1String("LABEL"))
+                return QVariant(d->m_channelLabel[channel]);
+            else if (key == QLatin1String("USED"))
+                return QVariant(d->m_channelUsed[channel]);
+        }
+        return QVariant();
+    }
+
+    void WinMIDIObject::sendInitialProgramChanges()
+    {
+        kDebug();
+        for (int i = 0; i < MIDI_CHANNELS; ++i) {
+            int patch(d->m_channelPatches[i]);
+            d->m_out->sendInitialProgram(i, patch);
+        }
+    }
 }
 
 #include "winmidiobject.moc"
