@@ -79,6 +79,7 @@ namespace KMid {
         {
             for (int chan = 0; chan < MIDI_CHANNELS; ++chan) {
                 m_lastpgm[chan] = 0;
+                m_lockedpgm[chan] = 0;
                 m_volumeShift[chan] = 1.0;
                 m_volume[chan] = 100;
                 m_muted[chan] = false;
@@ -132,9 +133,9 @@ namespace KMid {
 
         void transformProgramEvent(WinMIDIPacket &packet)
         {
+            int channel = packet.data[0] & MIDI_CHANNEL_MASK;
+            m_lastpgm[channel] = packet.data[1];
             if (m_mapper != NULL && m_mapper->isOK()) {
-                int channel = packet.data[0] & MIDI_CHANNEL_MASK;
-                m_lastpgm[channel] = packet.data[1];
                 int pgm = m_mapper->patch(channel, m_lastpgm[channel]);
                 if (pgm >= 0 && pgm < 128)
                     packet.data[1] = pgm;
@@ -195,6 +196,7 @@ namespace KMid {
         int m_currentOutput;
         QMap<int,QString> m_outputDevices;
         int m_lastpgm[MIDI_CHANNELS];
+        int m_lockedpgm[MIDI_CHANNELS];
         qreal m_volumeShift[MIDI_CHANNELS];
         int m_volume[MIDI_CHANNELS];
         bool m_muted[MIDI_CHANNELS];
@@ -278,6 +280,9 @@ namespace KMid {
     void inline WinMIDIOutput::outEvent(int &lastTick, bool &first, SongEvent *ev)
     {
         int i, j;
+        quint8 status;
+        quint8 chan;
+        bool reject(false);
         DWORD delta = 0;
         WinMIDIPacket packet, lpack;
         packet.dwPacket = 0;
@@ -286,7 +291,18 @@ namespace KMid {
         case SHORT_EVENT:
             packet.dwPacket = ev->getPacket();
             d->transform(packet);
-            packet.data[3] = MEVT_SHORTMSG;
+            status = packet.data[0] & MIDI_STATUS_MASK;
+            chan = packet.data[0] & MIDI_CHANNEL_MASK;
+            reject = ( ( status < MIDI_STATUS_NOTEOFF ) ||
+                       ( status >= MIDI_STATUS_SYSEX ) ||
+                       ((status < MIDI_STATUS_SYSEX) &&
+                         d->m_muted[chan]) ||
+                       ((status == MIDI_STATUS_PROGRAMCHANGE) &&
+                         d->m_locked[chan]) );
+            if (reject)
+                packet.dwPacket = 0;
+            else
+                packet.data[3] = MEVT_SHORTMSG;
             break;
         case TEMPO_EVENT:
             d->m_currentTempo = ev->getPacket();
@@ -589,6 +605,10 @@ namespace KMid {
         if (channel >= 0 && channel < MIDI_CHANNELS) {
             if (d->m_locked[channel] != lock) {
                 d->m_locked[channel] = lock;
+                if (lock) {
+                    d->m_lockedpgm[channel] = d->m_lastpgm[channel];
+                    kDebug() << channel << d->m_lastpgm[channel];
+                }
                 emit lockedChanged( channel, lock );
             }
         }
@@ -638,22 +658,22 @@ namespace KMid {
         }
     }
 
-    void WinMIDIOutput::sendShortMessage(WinMIDIPacket &packet)
+    void WinMIDIOutput::sendShortMessage(WinMIDIPacket &packet, bool discardable)
     {
         MMRESULT res;
         quint8 status;
         quint8 chan;
-        bool ok;
+        bool reject(false);
         d->transform(packet);
         status = packet.data[0] & MIDI_STATUS_MASK;
         chan = packet.data[0] & MIDI_CHANNEL_MASK;
-        ok = ( status >= MIDI_STATUS_NOTEOFF ) &&
-             ( ( status == MIDI_STATUS_SYSEX ) |
-               ( status < MIDI_STATUS_SYSEX &&
-                 !d->m_muted[chan] ) );
-        if ( ok && status == MIDI_STATUS_PROGRAMCHANGE )
-            ok = !d->m_locked[chan];
-        if (ok) {
+        reject = ( ( status < MIDI_STATUS_NOTEOFF ) ||
+                   ( status >= MIDI_STATUS_SYSEX ) ||
+                   ((status < MIDI_STATUS_SYSEX) &&
+                     d->m_muted[chan] && discardable) );
+        if ( discardable && (status == MIDI_STATUS_PROGRAMCHANGE) )
+            reject |= d->m_locked[chan];
+        if (!reject) {
             res = midiOutShortMsg( (HMIDIOUT)d->m_outHandle, packet.dwPacket );
             if ( res != MMSYSERR_NOERROR )
                 kDebug() << "midiOutShortMsg() err:" << mmErrorString(res);
@@ -875,6 +895,19 @@ namespace KMid {
         d->m_currentTempo = initialTempo;
         d->m_pendingTempo = true;
     }
+
+    void WinMIDIOutput::sendInitialProgram(int chan, int program)
+    {
+        int pgm(d->m_locked[chan] ? d->m_lockedpgm[chan] : program);
+        if (pgm > -1) {
+            kDebug() << chan << pgm;
+            WinMIDIPacket packet;
+            packet.data[0] = MIDI_STATUS_PROGRAMCHANGE | (chan & MIDI_CHANNEL_MASK);
+            packet.data[1] = pgm;
+            sendShortMessage(packet, false);
+        }
+    }
+
 }
 
 #include "winmidioutput.moc"
